# -*- coding: utf-8 -*-
"""
Created on Tue May  5 13:23:02 2020

@author: f-ove

script to try and use functions
"""
import time 
startTime = time.time()


import pandas as pd
import os
import numpy as np
import rawdata
# from .rawdata import pickle_rawdata
# from .rawdata import merge_frompickle


"""
for testing pickle and merge function
"""
# rawdata.pickle_rawdata(force_rerun=True)
# rawdata.merge_frompickle(force_rerun=True)


"""
load complete data
"""
data = rawdata.read_db()


"""
take a view at possible filter options
"""
# countrylist = rawdata.get_Countrylist()
# yearlist = rawdata.get_Yearlist()
# pollutantlist = rawdata.get_Pollutantlist()
# NACElist = rawdata.get_NACECode_filter_list()


"""
declaring filter
"""
# countryfilter = ['Germany','Spain','Belgium']
# yearfilter = [2016]
# mediumfilter = ['Air','Water']
pollutantfilter = ['Carbon dioxide (CO2)']
# NACEfilter = rawdata.get_NACECode_filter(group='chem')


"""
load data, with selected filters from above
"""
CO2 = rawdata.f_db(db=data,CountryName='Germany',PollutantName=pollutantfilter)
# CO2 = rawdata.f_db(nc=NACEfilter)


"""
get pollutant volume for chosen filters(all wanted filters have to be named in the input)
"""
# CO2 = rawdata.get_PollutantVolume(PollutantName=pollutantfilter,Table=False)
# CO2 = rawdata.get_PollutantVolume(PollutantName=pollutantfilter)


"""
plot pollutant volume for chosen filters(all wanted filters have to be named in the input)
"""
# rawdata.plot_PollutantVolume(CountryName=countryfilter,PollutantName=pollutantfilter,form=1)
# rawdata.plot_PollutantVolume(CountryName=countryfilter,PollutantName=pollutantfilter,ReportingYear=yearfilter,form=2)
# rawdata.plot_PollutantVolume(CountryName=countryfilter,PollutantName=pollutantfilter,ReportingYear=yearfilter,form=3)


print('The script took {0} second !'.format(time.time() - startTime))



# -*- coding: utf-8 -*-
"""
Created on Tue May 19 10:47:31 2020

@author: f-ove
"""

import pandas as pd
import os
from os.path import join, isfile

def read_db():
    """
    Reads complete database
    Returns the whole database as dataframe
    """
    try:
        db = pd.read_pickle(os.path.join(os.getcwd(), 'rawdata\\db.pkl'))
    except FileNotFoundError:
        print('db.pkl does not exist, please rerun merging routine')
    return db


def get_NACECode_filter_list():
    """
    returns list of industry sectors
    """
    NACElist = []
    NACElist.append(' Cement & Chalk: cem')
    NACElist.append(' Iron & Steel: is')
    NACElist.append(' Paper & Wood: pap')
    NACElist.append(' Chemistry: chem')
    NACElist.append(' Aluminium: alu')
    NACElist.append(' Refinery: ref')
    NACElist.append(' Glas: gla')
    NACElist.append(' Waste: wa')
    return NACElist


def get_NACECode_filter(group=None):
    """
    returns a list with the NACECode for preselected industry sectors (actual selection: Simon's list)
    """
    if group == 'cem':
        NACECode = ['23.51', '23.52']
    elif group == 'is':
        NACECode = ['19.10', '24.10', '24.20', '24.51', '24.52', '24.53', '24.54']
    elif group == 'pap':
        NACECode = ['16.21', '16.22', '16.23', '16.24', '16.29', '17.11', '17.12', '17.21', '17.22', '17.23', '17.24', '17.29']
    elif group == 'chem':
        NACECode = ['20.11', '20.12', '20.13', '20.14', '20.15', '20.16', '20.17', '20.20', '20.30', '20.41', '20.42', '20.51', '20.52', '20.53', '20.59', '21.10', '10.20', '22.11', '22.19', '22.21', '22.22', '22.23', '22.29']
    elif group == 'alu':
        NACECode = ['24.42']
    elif group == 'ref':
        NACECode = ['19.20']
    elif group == 'gla':
        NACECode = ['23.11', '23.12', '23.13', '23.14', '23.19']
    elif group == 'wa':
        NACECode = ['38.11', '38.12', '38.21', '38.22', '38.31', '38.32']
    return NACECode


def get_Countrylist():
    """
    Reads db.pkl
    Returns a list of all appearing countrys
    """
    Countrylist = []
    db = read_db()
    for items in db.CountryName.unique():
        Countrylist.append(items)
    return Countrylist


def get_Yearlist():
    """
    Reads db.pkl
    Returns a list of all appearing reporting years
    """
    Yearlist = []
    db = read_db()
    for items in db.ReportingYear.unique():
        Yearlist.append(items)
    return Yearlist


def get_Pollutantlist():
    """
    Reads db.pkl
    Returns a list of all appearing pollutants
    """
    Pollutantlist = []
    db = read_db()
    for items in db.PollutantName.unique():
        Pollutantlist.append(items)
    return Pollutantlist


def f_db(db, CountryName=None, ReportingYear=None, ReleaseMediumName=None, PollutantName=None, PollutantGroupName=None, NACEMainEconomicActivityCode=None):
    """
    filters db by country, year, release medium name, pollutant name, pollutantgroupname, NACEMainEconomicActivityCode

    To do: Think about excluding filters, not just wanted filters
    """
    if CountryName is not None:
        if isinstance(CountryName, list):
            db = db[db.CountryName.isin(CountryName)]
        else:
            db = db[db.CountryName == CountryName]

    if ReportingYear is not None:
        if isinstance(ReportingYear, list):
            db = db[db.ReportingYear.isin(ReportingYear)]
        else:
            db = db[db.ReportingYear == ReportingYear]

    if ReleaseMediumName is not None:
        if isinstance(ReleaseMediumName, list):
            db = db[db.ReleaseMediumName.isin(ReleaseMediumName)]
        else:
            db = db[db.ReleaseMediumName == ReleaseMediumName]

    if PollutantName is not None:
        if isinstance(PollutantName, list):
            db = db[db.PollutantName.isin(PollutantName)]
        else:
            db = db[db.PollutantName == PollutantName]

    if PollutantGroupName is not None:
        if isinstance(PollutantGroupName, list):
            db = db[db.PollutantGroupName.isin(PollutantGroupName)]
        else:
            db = db[db.PollutantGroupName == PollutantGroupName]

    if NACEMainEconomicActivityCode is not None:
        if isinstance(NACEMainEconomicActivityCode, list):
            db = db[db.NACEMainEconomicActivityCode.isin(NACEMainEconomicActivityCode)]
        else:
            db = db[db.NACEMainEconomicActivityCode == NACEMainEconomicActivityCode]

    return db


def rename_columns(db):
    """
    Change columnnames of db
    """
    db = db.rename(columns={'CountryName': 'Country'})
    db = db.rename(columns={'ReportingYear': 'Year'})
    return db


def row_reduction(db):
    """
    Reduces table to wanted rows
    To do: reduction to row names, that are given by input
    """
    db = db[['PollutantReleaseAndTransferReportID', 'CountryName', 'ReportingYear', 'FacilityReportID', 'PollutantReleaseID', 'ReleaseMediumName', 'PollutantName', 'PollutantGroupName', 'TotalQuantity', 'NACEMainEconomicActivityCode']]
    return db



# -*- coding: utf-8 -*-
"""
Created on Tue May  5 12:51:18 2020

@author: f-ove

all functions by Flo

TO DO: put them into organised .py scripts
"""

import pandas as pd
import os
from os.path import join, isfile
import matplotlib.pyplot as plt


def pickle_rawdata(force_rerun=False):
    """
    Write the following csv Sheets to Pickle Files:
    - POLLUTANTRELEASEANDTRANSFERREPORT
    - FACILITYREPORT
    - POLLUTANTRELEASE

    The script checks if the pickle files exist already.
    If it does, it skips the operation.

    If you set force_rerun to True, it will redo all operations.
    """

    # POLLUTANTRELEASEANDTRANSFERREPORT
    if ((os.path.isfile(os.path.join(os.getcwd(), 'rawdata\\pratr.pkl')) is False) or force_rerun):
        pratr = pd.read_csv(os.path.join(os.getcwd(), 'rawdata\\dbo.PUBLISH_POLLUTANTRELEASEANDTRANSFERREPORT.csv'))
        pratr.to_pickle(os.path.join(os.getcwd(), 'rawdata\\pratr.pkl'))

    # FACILITYREPORT
    if ((os.path.isfile(os.path.join(os.getcwd(), 'rawdata\\fr.pkl')) is False) or force_rerun):
        fr = pd.read_csv(os.path.join(os.getcwd(), 'rawdata\\dbo.PUBLISH_FACILITYREPORT.csv'), encoding='latin-1', low_memory=False)
        fr.to_pickle(os.path.join(os.getcwd(), 'rawdata\\fr.pkl'))

    # POLLUTANTRELEASE
    if ((os.path.isfile(os.path.join(os.getcwd(), 'rawdata\\pr.pkl')) is False) or force_rerun):
        pr = pd.read_csv(os.path.join(os.getcwd(), 'rawdata\\dbo.PUBLISH_POLLUTANTRELEASE.csv'), low_memory=False)
        pr.to_pickle(os.path.join(os.getcwd(), 'rawdata\\pr.pkl'))
    return None


def merge_frompickle(force_rerun=False):
    """
    This function will merge the desired data to one dataframe and stores it as pickle

    The script checks if the pickle files exist already.
    If it does, it skips the operation.

    If you set force_rerun to True, it will redo all operations.
    """
    if (os.path.isfile(os.path.join(os.getcwd(), 'rawdata\\db.pkl')) is False) or force_rerun:
        try:
            fr = pd.read_pickle(os.path.join(os.getcwd(), 'rawdata\\fr.pkl'))
            pr = pd.read_pickle(os.path.join(os.getcwd(), 'rawdata\\pr.pkl'))
            pratr = pd.read_pickle(os.path.join(os.getcwd(), 'rawdata\\pratr.pkl'))
        except FileNotFoundError:
            print('Error. Run function pickle_rawdata')

        try:
            # speed difference for variing merge-order?? Table length differs, merge smart enough to add multiple one row to multiple?
            # problematic to merge by multiple coloum names?
            db01 = pd.merge(fr, pratr, on=['PollutantReleaseAndTransferReportID', 'CountryName', 'CountryCode'])
            db02 = pd.merge(db01, pr, on=['FacilityReportID', 'ConfidentialIndicator', 'ConfidentialityReasonCode', 'ConfidentialityReasonName'])
            db02.to_pickle(os.path.join(os.getcwd(), 'rawdata\\db.pkl'))
        except NameError:
            print('Error. Merging not possible')
    return None















# -*- coding: utf-8 -*-
"""
Created on Tue May 19 10:45:28 2020

@author: f-ove

Functions for pickle editing
"""


import pandas as pd
import os
from os.path import join, isfile
import matplotlib.pyplot as plt




def get_PollutantVolume(db, Table=True):
    """
    Returns the Volume of a Pollutant
    if Table is True it returns dataframe, if Table is False it returns Integer

    Really necessary? if table is not put false, it just returns db...

    """
    if Table is True:
        return db
    else:
        PollutantVolume = db['TotalQuantity'].sum()
        return PollutantVolume


def plot_PollutantVolume(db, form=1):
    """
    Plots the Pollutant Volume to wanted category.
    form = 1: against year
    form = 2: against country
    form = 3: against pollutant
    """

    # Volume against ReportingYear
    if form == 1:
        db = db.groupby(['ReportingYear']).sum().reset_index()
        db.plot(x='ReportingYear', y='TotalQuantity', kind='bar')
    # Volume against CountryName
    elif form == 2:
        db = db.groupby(['CountryName']).sum().reset_index()
        db.plot(x='CountryName', y='TotalQuantity', kind='bar')
    # Volume against PollutantName
    else:
        db = db.groupby(['PollutantName']).sum().reset_index()
        db.plot(x='PollutantName', y='TotalQuantity', kind='bar')

